@@ -4,7 +4,10 @@
 /* Written by Olaf Matthes <olaf.matthes@gmx.de>.                               */
 /* Get source at http://www.akustische-kunst.org/                               */
 /*                                                                              */
-/* This program is free software; you can redistribute it and/or                */
+/* Based on freeverb by Jeremy "Jezar" Wakefield http://www.dreampoint.co.uk    */
+/* Freeverb is Public Domain software. The Pd/Max/MSP portions are GPL 2 or up. */
+/*                                                                              */
+/* Parts of this program are free software; you can redistribute it and/or      */
 /* modify it under the terms of the GNU General Public License                  */
 /* as published by the Free Software Foundation; either version 2               */
 /* of the License, or (at your option) any later version.                       */
@@ -77,7 +80,7 @@ static const int allpasstuningL[numallpasses]
 static const int allpasstuningR[numallpasses]
                      = { 556+stereospread, 441+stereospread, 341+stereospread, 225+stereospread };
 
-static char *version = "freeverb~ v1.2";
+static char *version = "freeverb~ v1.2.3";
 
 #ifdef PD
 static t_class *freeverb_class;
@@ -128,6 +131,7 @@ typedef struct _freeverb
 
 	int x_allpasstuningL[numallpasses];
 	int x_allpasstuningR[numallpasses];
+    int x_firstpassfilter;
 
 #ifdef PD
 	t_float x_float;
@@ -222,17 +226,20 @@ static inline t_float comb_processL(t_freeverb *x, int filteridx, t_float input)
 
 	output = x->x_bufcombL[filteridx][bufidx];
     //FIX_DENORM_NAN_FLOAT(output);
-    fix_denorm_nan_float(output);
+    output = fix_denorm_nan_float(output);
 
     x->x_filterstoreL[filteridx] = (output*x->x_combdamp2) + (x->x_filterstoreL[filteridx]*x->x_combdamp1);
     //FIX_DENORM_NAN_FLOAT(x->x_filterstoreL[filteridx]);
-    fix_denorm_nan_float(x->x_filterstoreL[filteridx]);
+    x->x_filterstoreL[filteridx] = fix_denorm_nan_float(x->x_filterstoreL[filteridx]);
 
 	x->x_bufcombL[filteridx][bufidx] = input + (x->x_filterstoreL[filteridx]*x->x_combfeedback);
 
 	if(++x->x_combidxL[filteridx] >= x->x_combtuningL[filteridx]) x->x_combidxL[filteridx] = 0;
 
-	return output;
+    if (x->x_firstpassfilter) 
+        return x->x_filterstoreL[filteridx];
+    else
+        return output;
 }
 
 static inline t_float comb_processR(t_freeverb *x, int filteridx, t_float input)
@@ -242,17 +249,20 @@ static inline t_float comb_processR(t_freeverb *x, int filteridx, t_float input)
 
 	output = x->x_bufcombR[filteridx][bufidx];
     //FIX_DENORM_NAN_FLOAT(output);
-    fix_denorm_nan_float(output);
+    output = fix_denorm_nan_float(output);
 
 	x->x_filterstoreR[filteridx] = (output*x->x_combdamp2) + (x->x_filterstoreR[filteridx]*x->x_combdamp1);
     //FIX_DENORM_NAN_FLOAT(x->x_filterstoreR[filteridx]);
-    fix_denorm_nan_float(x->x_filterstoreR[filteridx]);
+    x->x_filterstoreR[filteridx] = fix_denorm_nan_float(x->x_filterstoreR[filteridx]);
 
 	x->x_bufcombR[filteridx][bufidx] = input + (x->x_filterstoreR[filteridx]*x->x_combfeedback);
 
 	if(++x->x_combidxR[filteridx] >= x->x_combtuningR[filteridx]) x->x_combidxR[filteridx] = 0;
 
-	return output;
+    if (x->x_firstpassfilter) 
+        return x->x_filterstoreR[filteridx];
+    else
+        return output;
 }
 
 /* -------------------- allpass filter stuff ----------------------- */
@@ -270,7 +280,7 @@ static inline t_float allpass_processL(t_freeverb *x, int filteridx, t_float inp
 	
 	bufout = (t_float)x->x_bufallpassL[filteridx][bufidx];
     //FIX_DENORM_NAN_FLOAT(bufout);
-    fix_denorm_nan_float(bufout);
+    bufout = fix_denorm_nan_float(bufout);
 	
 	output = -input + bufout;
 	x->x_bufallpassL[filteridx][bufidx] = input + (bufout*x->x_allpassfeedback);
@@ -289,7 +299,7 @@ static inline t_float allpass_processR(t_freeverb *x, int filteridx, t_float inp
 	
 	bufout = (t_float)x->x_bufallpassR[filteridx][bufidx];
     //FIX_DENORM_NAN_FLOAT(bufout);
-    fix_denorm_nan_float(bufout);
+    bufout = fix_denorm_nan_float(bufout);
 	
 	output = -input + bufout;
 	x->x_bufallpassR[filteridx][bufidx] = input + (bufout*x->x_allpassfeedback);
@@ -686,9 +696,14 @@ static float freeverb_getdb(float f)
     }
 }
 
+static void freeverb_setfirstpassfilter(t_freeverb *x, t_float fpf)
+{
+    x->x_firstpassfilter = (fpf > 0) ? 1 : 0;
+}
+
 static void freeverb_print(t_freeverb *x)
 {
-	post("freeverb~:");
+	post("%s:", version);
 	if(x->x_bypass) {
 		post("  bypass: on");
 	} else post("  bypass: off");
@@ -700,6 +715,7 @@ static void freeverb_print(t_freeverb *x)
 	post("  width: %g %%", x->x_width * 100);
 	post("  wet level: %g dB", freeverb_getdb(freeverb_getwet(x)*scalewet));
 	post("  dry level: %g dB", freeverb_getdb(freeverb_getdry(x)*scaledry));
+	post("  firstpassfilter: %s", (x->x_firstpassfilter) ? "on" : "off");
 }
 
 	// clean up
@@ -793,6 +809,7 @@ void *freeverb_new(t_floatarg f)
 
 	// buffers will be full of rubbish - so we MUST mute them
 	freeverb_mute(x);
+	x->x_firstpassfilter = 0;
 
     return (x);
 }
@@ -813,7 +830,8 @@ void freeverb_tilde_setup(void)
 	class_addmethod(freeverb_class, (t_method)freeverb_setbypass, gensym("bypass"), A_FLOAT, A_NULL);
 	class_addmethod(freeverb_class, (t_method)freeverb_mute, gensym("clear"), A_NULL);
     class_addmethod(freeverb_class, (t_method)freeverb_print, gensym("print"), A_NULL);
-	post(version);
+    class_addmethod(freeverb_class, (t_method)freeverb_setfirstpassfilter, gensym("firstpassfilter"), A_FLOAT, A_NULL);
+	logpost(NULL, 4, version);
 }
 
 #else
